=== modified file 'NEWS'
--- NEWS	2007-02-22 14:28:19 +0000
+++ NEWS	2007-03-03 05:47:31 +0000
@@ -130,6 +130,9 @@
       before the rest of the suite.  (Martin Pool)
 
 
+    * Fix bzrtools shelve command for removed lines beginning with "--"
+      (Johan Dahlberg, #75577)
+
 bzr 0.14  2007-01-23
 
   IMPROVEMENTS:

=== modified file 'bzrlib/patches.py'
--- bzrlib/patches.py	2006-10-11 23:32:09 +0000
+++ bzrlib/patches.py	2007-03-03 05:47:31 +0000
@@ -316,15 +316,22 @@
 
 def iter_file_patch(iter_lines):
     saved_lines = []
+    orig_range = 0
     for line in iter_lines:
         if line.startswith('=== ') or line.startswith('*** '):
             continue
         if line.startswith('#'):
             continue
+        elif orig_range > 0:
+            if line.startswith('-') or line.startswith(' '):
+                orig_range -= 1
         elif line.startswith('--- '):
             if len(saved_lines) > 0:
                 yield saved_lines
             saved_lines = []
+        elif line.startswith('@@'):
+            hunk = hunk_from_header(line)
+            orig_range = hunk.orig_range
         saved_lines.append(line)
     if len(saved_lines) > 0:
         yield saved_lines

=== modified file 'bzrlib/tests/test_patches.py'
--- bzrlib/tests/test_patches.py	2006-10-11 23:32:09 +0000
+++ bzrlib/tests/test_patches.py	2007-03-03 05:47:31 +0000
@@ -31,7 +31,8 @@
                             hunk_from_header, 
                             iter_patched, 
                             parse_line,
-                            parse_patch)
+                            parse_patch,
+                            parse_patches)
 
 
 class PatchesTester(unittest.TestCase):
@@ -182,6 +183,42 @@
         patch = parse_patch(self.datafile("insert_top.patch"))
         assert (patch.pos_in_mod(0)==1)
 
+    def testParsePatches(self):
+        """Make sure file names can be extracted from tricky unified diffs"""
+        patchtext = \
+"""--- orig-7
++++ mod-7
+@@ -1,10 +1,10 @@
+ -- a
+--- b
++++ c
+ xx d
+ xx e
+ ++ f
+-++ g
++-- h
+ xx i
+ xx j
+ -- k
+--- l
++++ m
+--- orig-8
++++ mod-8
+@@ -1 +1 @@
+--- A
++++ B
+@@ -1 +1 @@
+--- C
++++ D
+"""
+        filenames = [('orig-7', 'mod-7'),
+                     ('orig-8', 'mod-8')]
+        patches = parse_patches(patchtext.splitlines(True))
+        patch_files = []
+        for patch in patches:
+            patch_files.append((patch.oldname, patch.newname))
+        assert (patch_files == filenames)
+            
 def test():
     patchesTestSuite = unittest.makeSuite(PatchesTester,'test')
     runner = unittest.TextTestRunner(verbosity=0)

